@@ -3,7 +3,7 @@
  *
  * This program is licensed under BSD license, read COPYING
  *
- *  $Id: x11.c,v 1.10 2005/09/28 03:21:10 brenden1 Exp $
+ *  $Id: x11.c,v 1.14 2005/12/31 17:36:16 brenden1 Exp $
  */
 
 
@@ -17,6 +17,8 @@
 #include <X11/Xft/Xft.h>
 #endif
 
+#include <stdio.h>
+
 #ifdef XDBE
 int use_xdbe;
 #endif
@@ -25,6 +27,8 @@ int use_xdbe;
 int use_xft = 0;
 #endif
 
+#define WINDOW_NAME_FMT "%s - conky" 
+
 /* some basic X11 stuff */
 Display *display;
 int display_width;
@@ -171,9 +175,9 @@ inline void set_transparent_background(Window win)
 }
 
 #if defined OWN_WINDOW
-void init_window(int own_window, int w, int h, int l, int fixed_pos, int set_trans, int back_colour)
+void init_window(int own_window, char* wm_class_name, int w, int h, int l, int fixed_pos, int set_trans, int back_colour, char * nodename)
 #else
-void init_window(int own_window, int w, int h, int l, int set_trans, int back_colour)
+void init_window(int own_window, int w, int h, int l, int set_trans, int back_colour, char * nodename)
 #endif
 {
 	/* There seems to be some problems with setting transparent background (on
@@ -181,6 +185,7 @@ void init_window(int own_window, int w, int h, int l, int set_trans, int back_co
 	 * happens but I bet the bug is somewhere here. */
 	set_transparent = set_trans;
 	background_colour = back_colour;
+	char * window_name = NULL;
 #ifdef OWN_WINDOW
 	if (own_window) {
 
@@ -207,15 +212,16 @@ void init_window(int own_window, int w, int h, int l, int set_trans, int back_co
 
 
 
-			class_hints.res_class = "conky";
-			class_hints.res_name = "conky";
+			class_hints.res_class = wm_class_name;
+			class_hints.res_name = wm_class_name;
 			XSetClassHint(display, window.window,
 				      &class_hints);
 
 			/*set_transparent_background(window.window);*/
-
-			XStoreName(display, window.window, "conky");
-
+			window_name = (char *) malloc(strlen(WINDOW_NAME_FMT) + strlen(nodename)+1);
+		        sprintf(window_name, WINDOW_NAME_FMT, nodename);
+			XStoreName(display, window.window, window_name);
+			free(window_name);
 			XClearWindow(display, window.window);
 
 			if (!fixed_pos)
@@ -360,8 +366,17 @@ long get_x11_color(const char *name)
 	color.pixel = 0;
 	if (!XParseColor
 	    (display, DefaultColormap(display, screen), name, &color)) {
-		ERR("can't parse X color '%s'", name);
-		return 0xFF00FF;
+		/* lets check if it's a hex colour with the # missing in front
+		 * if yes, then do something about it
+		 */
+		char newname[64];
+		newname[0] = '#';
+		strncpy(&newname[1], name, 62);
+		/* now lets try again */
+		if (!XParseColor(display, DefaultColormap(display, screen), &newname[0], &color)) {
+			ERR("can't parse X color '%s'", name);
+			return 0xFF00FF;
+		}
 	}
 	if (!XAllocColor
 	    (display, DefaultColormap(display, screen), &color))
